Pixel

Enum Pixel 

pub enum Pixel {
    D2([usize; 2]),
    D3([usize; 3]),
}
Position of a pixel position in an Image.

Variants



D2([usize; 2])

Index in a 2-dimensional image.

Fields are [row, column].



D3([usize; 3])

Index in a 3-dimensional image.

Fields are [slice, row, column].

Implementations

Source

impl Pixel

pub fn from_slice(pos: &[usize]) -> Self

Construct a pixel position from a slice of indices.

Panics

pos must contain exactly 2 or 3 elements, respectively indicating a 2-dimensional or 3-dimensional position. This function will panic if another length is found.