Image

Struct Image 

pub struct Image<T = ()>(/* private fields */);
Native Wolfram ImageWL or Image3DWL.

Use UninitImage::new_2d() to construct a new 2-dimensional image.

Implementations

Source

impl<T: ImageData> Image<T>

pub fn as_slice(&self) -> &[T::STORAGE]

Access the data in this Image as a flat buffer.

The returned slice will have a length equal to flattened_length().

Examples found in repository?
examples/tests/test_images.rs (line 11)
7fn test_image_arg(image: &Image<bool>) -> NumericArray<i8> {
8    let mut array = UninitNumericArray::from_dimensions(&[image.flattened_length()]);
9
10    for pair in image
11        .as_slice()
12        .iter()
13        .zip(array.as_slice_mut().into_iter())
14    {
15        let (pixel, elem): (&i8, &mut std::mem::MaybeUninit<i8>) = pair;
16        elem.write(*pixel);
17    }
18
19    // Safety: We iterated over every element in `array` and initialized it with the
20    //         corresponding pixel value from `image`.
21    unsafe { array.assume_init() }
22}
pub fn get(&self, pixel: Pixel, channel: usize) -> Option<T>

Get the value of the specified pixel and channel.

Example

Get the value of the second channel of the top-left pixel in an image.

// let image: Image<u8> = ...

let value: u8 = image.get(Pixel::D2([0, 0]), 2).unwrap();

In an RGB image, this is the value of the green channel for this pixel.

In an HSB image, this is the value of the saturation for this pixel.

Source

impl<T> Image<T>

pub fn flattened_length(&self) -> usize

The number of elements in the underlying flat data buffer.

LibraryLink C API Documentation: MImage_getFlattenedLength

Examples found in repository?
examples/tests/test_images.rs (line 8)
7fn test_image_arg(image: &Image<bool>) -> NumericArray<i8> {
8    let mut array = UninitNumericArray::from_dimensions(&[image.flattened_length()]);
9
10    for pair in image
11        .as_slice()
12        .iter()
13        .zip(array.as_slice_mut().into_iter())
14    {
15        let (pixel, elem): (&i8, &mut std::mem::MaybeUninit<i8>) = pair;
16        elem.write(*pixel);
17    }
18
19    // Safety: We iterated over every element in `array` and initialized it with the
20    //         corresponding pixel value from `image`.
21    unsafe { array.assume_init() }
22}
pub fn channels(&self) -> usize

LibraryLink C API Documentation: MImage_getChannels

pub fn rank(&self) -> usize

LibraryLink C API Documentation: MImage_getRank

pub fn row_count(&self) -> usize

LibraryLink C API Documentation: MImage_getRowCount

pub fn column_count(&self) -> usize

LibraryLink C API Documentation: MImage_getColumnCount

pub fn slice_count(&self) -> usize

LibraryLink C API Documentation: MImage_getSliceCount

pub fn color_space(&self) -> ColorSpace

Get the color space of this image.

LibraryLink C API Documentation: MImage_getColorSpace

pub fn color_space_raw(&self) -> colorspace_t

LibraryLink C API Documentation: MImage_getColorSpace

pub fn data_type(&self) -> ImageType

Get the data type of this image.

pub fn data_type_raw(&self) -> imagedata_t

LibraryLink C API Documentation: MImage_getDataType

pub fn has_alpha_channel(&self) -> bool

LibraryLink C API Documentation: MImage_alphaChannelQ

pub fn is_interleaved(&self) -> bool

LibraryLink C API Documentation: MImage_interleavedQ

pub fn share_count(&self) -> usize

Returns the share count of this Image.

If this Image is not shared, the share count is 0.

If this Image was passed into the current library “by reference” due to use of the Automatic or "Constant" memory management strategy, that reference is not reflected in the share_count().

LibraryLink C API Documentation: MImage_shareCount

pub unsafe fn from_raw(raw: MImage) -> Image<T>

Construct an Image from a raw MImage.

pub unsafe fn into_raw(self) -> MImage

Extract the raw MImage instance from this Image.

pub unsafe fn as_raw(&self) -> MImage

pub unsafe fn raw_data(&self) -> *mut c_void

LibraryLink C API Documentation: MImage_getRawData