pub enum LibraryError {
RustPanic {
message: String,
source_location: String,
backtrace: Expr,
},
Loader {
message: String,
expected: String,
got: Expr,
},
WstpError {
code: i32,
message: String,
},
WstpErrorMessage {
message: String,
},
}Expand description
An error raised at the LibraryLink boundary.
#[derive(Failure)] renders each variant to its Failure["VariantName", <|CamelCase fields|>] expression (e.g. RustPanic { message, .. } →
Failure["RustPanic", <|"Message" -> …, "SourceLocation" -> …, "Backtrace" -> …|>]).
Variants
RustPanic
A Rust panic caught while running an exported function. The backtrace
is a renderable Expr (a clickable Column of frames when the
panic-failure-backtraces feature is on and the backtrace env var is
set, else Missing[…]).
Fields
Loader
The generated generate_loader! entry point was called incorrectly
(wrong head / argument count / argument type).
Fields
WstpError
A WSTP error with an error code.
WstpErrorMessage
A WSTP error without an error code.