pub enum ManagedExpressionEvent {
Create(Id),
Drop(Id),
}Expand description
Lifecycle events triggered by the creation and deallocation of managed expressions.
Variants
Create(Id)
Instruction that the library should create a new instance of a managed expression
with the specified Id.
This event occurs when CreateManagedLibraryExpression is called.
Drop(Id)
Instruction that the library should drop any data associated with the managed
expression identified by this Id.
This event occurs when the managed expression is no longer used by the Wolfram Language.
Implementations